@@ -11,6 +11,9 @@
  *
  */
 
+#include <config.h>
+
+#include <math.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -71,10 +74,10 @@ typedef struct {
 } GFIFontData;
 
 static void gfi_verify_fontmap (GFFontMap *map);
-static void gfi_verify_font_entry (GFFontEntry *e);
-static void gfi_verify_afm_file (GFFontEntry *e, GFFileEntry *f, gboolean force);
-static void gfi_verify_pfb_file (GFFontEntry *e, GFFileEntry *f);
-static void gfi_verify_ttf_file (GFFontEntry *e, GFFileEntry *f);
+static void gfi_verify_font_entry (GFFontEntry *e, gboolean forced);
+static void gfi_verify_afm_file (GFFontEntry *e, GFFileEntry *f, gboolean forced);
+static void gfi_verify_pfb_file (GFFontEntry *e, GFFileEntry *f, gboolean forced);
+static void gfi_verify_ttf_file (GFFontEntry *e, GFFileEntry *f, gboolean forced);
 static GFIFileData *gfi_read_afm_file_data (const guchar *name);
 static GFIFileData *gfi_read_pfb_file_data (const guchar *name);
 static void gfi_read_and_register_ttf_file_data (const guchar *name);
@@ -123,6 +126,7 @@ static gboolean gfi_debug = FALSE;
 static gboolean gfi_smart = FALSE;
 static gboolean gfi_recursive = FALSE;
 static gboolean gfi_clean = FALSE;
+static gboolean gfi_refresh = FALSE;
 static gboolean gfi_notice = FALSE;
 static gboolean gfi_usermap = TRUE;
 static gboolean gfi_dynamicmap = FALSE;
@@ -141,6 +145,8 @@ static const struct poptOption options[] = {
 	  N_("Search directories recursively for font files"), NULL },
 	{ "clean", 'c', POPT_ARG_NONE, &gfi_clean, 0,
 	  N_("Start from zero fontmap, instead of parsing old one"), NULL },
+	{ "refresh", 0, POPT_ARG_NONE, &gfi_refresh, 0,
+	  N_("Reverifies all font files, even for fontmaps of same version"), NULL },
 	{ "notice", 'n', POPT_ARG_NONE, &gfi_notice, 0,
 	  N_("Include font notice field (Copyright data) into fontmap"), NULL },
 	{ "user", 'u', POPT_ARG_NONE, &gfi_usermap, 0,
@@ -165,6 +171,8 @@ static const struct poptOption options[] = {
 };
 
 /* Well-known font paths */
+/* It is safe to add as many more as you want */
+
 static const guchar *smart_paths[] = {
         /* generic */
         "/usr/share/fonts",
@@ -195,9 +203,19 @@ static const guchar *smart_paths[] = {
         "/usr/X11R6/lib/X11/fonts",
 	/* Misc */
 	"/usr/share/abisource/fonts",
-	"/usr/share/AbiSuite/fonts"
+	"/usr/share/AbiSuite/fonts",
+	NULL
 };
-#define NUM_SMART_PATHS (sizeof (smart_paths) / sizeof (smart_paths[0]))
+
+static const guchar *smart_aliases[] = {
+	DATADIR "/gnome/libgnomeprint-2.0/fonts/adobe-urw.font",
+	DATADIR "/gnome/fonts/adobe-urw.font",
+	DATADIR "/fonts/adobe-urw.font",
+	"/usr/share/gnome/libgnomeprint-2.0/fonts/adobe-urw.font",
+	"/usr/share/gnome/fonts/adobe-urw.font",
+	"/usr/share/fonts/adobe-urw.font",
+	NULL
+ };
 
 static GSList *aliaspath_list = NULL;
 static GSList *default_list = NULL;
@@ -224,7 +242,8 @@ static GFFontMap *mastermap, *newmap;
 
 FT_Library ft_library;
 
-int main (int argc, char ** argv)
+int
+main (int argc, const char ** argv)
 {
 	poptContext ctx;
 	FT_Error ft_result;
@@ -276,16 +295,15 @@ int main (int argc, char ** argv)
 	if (gfi_smart) {
 		uid_t euid;
 		/* Set appropriate mode */
+		gfi_staticmap = gfi_dynamicmap = gfi_usermap = FALSE;
 		euid = geteuid ();
 		if (!euid) {
 			/* root - use dynamic map */
-			if (!gfi_staticmap) gfi_dynamicmap = TRUE;
-			gfi_usermap = FALSE;
+			gfi_dynamicmap = TRUE;
 		} else {
-			gfi_staticmap = FALSE;
-			gfi_dynamicmap = FALSE;
 			gfi_usermap = TRUE;
 		}
+		gfi_staticmap = FALSE;
 		/* Set recursive */
 		gfi_recursive = TRUE;
 		/* Not clean */
@@ -293,7 +311,9 @@ int main (int argc, char ** argv)
 		/* Set target */
 		gfi_target = NULL;
 		/* Set aliases */
-		aliaspath_list = g_slist_prepend (aliaspath_list, g_strdup (FONTMAPDIR_STATIC "/adobe-urw.font"));
+		for (i = 0; smart_aliases[i]; i++) {
+			aliaspath_list = g_slist_prepend (aliaspath_list, g_strdup (smart_aliases[i]));
+		}
 	}
 
 	/* Step 1: Read existing fontmap */
@@ -318,7 +338,7 @@ int main (int argc, char ** argv)
 	} else {
 		mastermap = db->usermaps;
 		newmap = gf_fontmap_new (GF_FONTMAP_USER, mastermap->path);
-		if (!gfi_target) gfi_target = g_strdup_printf ("%s/.gnome/fonts/gnome-print.fontmap", g_get_home_dir ());
+		if (!gfi_target) gfi_target = g_build_filename (g_get_home_dir (), ".gnome/fonts/gnome-print.fontmap", NULL);
 	}
 	g_assert (mastermap != NULL);
 
@@ -332,7 +352,7 @@ int main (int argc, char ** argv)
 	if (gfi_debug) fprintf (stderr, "Done\n");
 
 	/* Process alias files */
-	if (gfi_debug) fprintf (stderr, "Scanning alias maps... ");
+	if (gfi_debug) fprintf (stderr, "Scanning alias maps...\n");
 	while (aliaspath_list) {
 		gfi_read_aliases ((guchar *) aliaspath_list->data);
 		g_free (aliaspath_list->data);
@@ -348,7 +368,7 @@ int main (int argc, char ** argv)
 	}
 	if (gfi_smart) {
 		if (gfi_debug) fprintf (stderr, "Scanning smart paths\n");
-		for (i = 0; i < NUM_SMART_PATHS; i++) {
+		for (i = 0; smart_paths[i]; i++) {
 			if (gfi_debug) fprintf (stderr, "Scanning path %s\n", smart_paths[i]);
 			gfi_scan_path (smart_paths[i], 0);
 		}
@@ -377,7 +397,7 @@ int main (int argc, char ** argv)
 	 *
 	 */
 
-	if (gfi_debug) fprintf (stderr, "Building fonts... ");
+	if (gfi_debug) fprintf (stderr, "Building fonts...\n");
 	gfi_build_fonts ();
 	if (gfi_debug) fprintf (stderr, "Done\n");
 
@@ -414,9 +434,11 @@ gfi_verify_fontmap (GFFontMap *map)
 {
 	while (map) {
 		GFFontEntry *e;
-		if (gfi_debug) fprintf (stderr, "Map: %s ", map->path);
+		gint ourversion;
+		ourversion = (gint) floor (1000.0 * atof (VERSION) + 0.5);
+		if (gfi_debug) fprintf (stderr, "Map [%g]: %s ", (gdouble) map->gpversion / 1000.0, map->path);
 		for (e = map->fonts; e != NULL; e = e->next) {
-			gfi_verify_font_entry (e);
+			gfi_verify_font_entry (e, gfi_refresh || (map->gpversion < ourversion));
 			if (gfi_debug) fprintf (stderr, ".");
 		}
 		if (gfi_debug) fprintf (stderr, "\n");
@@ -434,11 +456,11 @@ gfi_verify_fontmap (GFFontMap *map)
  */
 
 static void
-gfi_verify_font_entry (GFFontEntry *e)
+gfi_verify_font_entry (GFFontEntry *e, gboolean forced)
 {
 	switch (e->type) {
 	case GF_FONT_ENTRY_TYPE1:
-		if (e->files[1].psname && strcmp (e->files[1].psname, e->files[0].psname)) {
+		if (e->isalias) {
 			GFIAliasData *a;
 			/* We are aliased entry */
 			a = g_new (GFIAliasData, 1);
@@ -450,16 +472,16 @@ gfi_verify_font_entry (GFFontEntry *e)
 			a->alias = g_strdup (e->files[0].psname);
 			alias_list = g_slist_prepend (alias_list, a);
 			/* Verify pfb as usual */
-			gfi_verify_pfb_file (e, &e->files[0]);
+			gfi_verify_pfb_file (e, &e->files[0], forced);
 			/* Forced verify of afm */
 			if (e->files[1].path) gfi_verify_afm_file (e, &e->files[1], TRUE);
 		} else {
-			gfi_verify_pfb_file (e, &e->files[0]);
+			gfi_verify_pfb_file (e, &e->files[0], forced);
 			if (e->files[1].path) gfi_verify_afm_file (e, &e->files[1], FALSE);
 		}
 		break;
 	case GF_FONT_ENTRY_TRUETYPE:
-		gfi_verify_ttf_file (e, &e->files[0]);
+		gfi_verify_ttf_file (e, &e->files[0], forced);
 		break;
 	default:
 		g_assert_not_reached ();
@@ -512,7 +534,7 @@ gfi_verify_afm_file (GFFontEntry *e, GFFileEntry *f, gboolean forced)
 /* Same as previous for pfb files */
 
 static void
-gfi_verify_pfb_file (GFFontEntry *e, GFFileEntry *f)
+gfi_verify_pfb_file (GFFontEntry *e, GFFileEntry *f, gboolean forced)
 {
 	GFIFileData * fd;
 
@@ -520,7 +542,7 @@ gfi_verify_pfb_file (GFFontEntry *e, GFFileEntry *f)
 	fd = g_hash_table_lookup (goodpfb_dict, f->path);
 	if (fd) return;
 
-	if (!gfi_test_file_changed (f)) {
+	if (!forced && !gfi_test_file_changed (f)) {
 		fd = g_new0 (GFIFileData, 1);
 		fd->type = GFI_FILE_PFB;
 		fd->entry.path = g_strdup (f->path);
@@ -547,7 +569,7 @@ gfi_verify_pfb_file (GFFontEntry *e, GFFileEntry *f)
 /* Same as previous for ttf files */
 
 static void
-gfi_verify_ttf_file (GFFontEntry *e, GFFileEntry *f)
+gfi_verify_ttf_file (GFFontEntry *e, GFFileEntry *f, gboolean forced)
 {
 	GFIFileData *fd;
 	gchar *key;
@@ -557,7 +579,7 @@ gfi_verify_ttf_file (GFFontEntry *e, GFFileEntry *f)
 	fd = g_hash_table_lookup (goodttf_dict, key);
 	if (fd) return;
 
-	if (!gfi_test_file_changed (f)) {
+	if (!forced && !gfi_test_file_changed (f)) {
 		fd = g_new0 (GFIFileData, 1);
 		fd->type = GFI_FILE_TTF;
 		fd->entry.path = g_strdup (f->path);
@@ -803,7 +825,7 @@ gfi_scan_path (const guchar *path, gint level)
 					while ((dent = readdir (dir))) {
 						gchar *fn;
 						if (!strcmp (dent->d_name, ".") || !strcmp (dent->d_name, "..")) continue;
-						fn = g_strdup_printf ("%s/%s", path, dent->d_name);
+						fn = g_build_filename (path, dent->d_name, NULL);
 						if (!stat (fn, &s)) {
 							if (S_ISREG (s.st_mode)) {
 								gfi_try_font_file (fn);
@@ -845,7 +867,7 @@ gfi_try_font_file (const guchar *fn)
 	if (!g_path_is_absolute (fn)) {
 		gchar *cdir;
 		cdir = g_get_current_dir ();
-		name = g_strdup_printf ("%s/%s", cdir, fn);
+		name = g_build_filename (cdir, fn, NULL);
 	} else {
 		name = g_strdup (fn);
 	}
@@ -964,6 +986,7 @@ gfi_process_alias (GFIAliasData *a)
 	new = g_new0 (GFFontEntry, 1);
 	new->next = NULL;
 	new->type = GF_FONT_ENTRY_TYPE1;
+	new->isalias = TRUE;
 	new->name = g_strdup (a->name);
 	new->version = g_strdup (a->version);
 	new->familyname = g_strdup (a->familyname);
@@ -972,8 +995,10 @@ gfi_process_alias (GFIAliasData *a)
 	new->weight = g_strdup (fe->weight);
 	new->italicangle = fe->italicangle;
 	/* AFM */
-	new->files[1] = fe->files[1];
-	new->files[1].path = g_strdup (new->files[1].path);
+	if (fe->files[1].path) {
+		new->files[1] = fe->files[1];
+		new->files[1].path = g_strdup (new->files[1].path);
+	}
 	/* fixme: This is hackish, but gos for now */
 	new->files[1].psname = g_strdup (a->psname);
 	/* PFB */
@@ -986,113 +1011,6 @@ gfi_process_alias (GFIAliasData *a)
 
 	goodfont_list = g_slist_prepend (goodfont_list, new);
 	g_hash_table_insert (goodfont_dict, new->name, new);
-#if 0
-	GFIFontData *afmfd, *pfbfd;
-	GFIFileData *afm, *pfb;
-	GSList *l;
-	gdouble afmversion, pfbversion;
-	FILE *f;
-	int status;
-	Font_Info *fi;
-
-	/* Return if we are already registered */
-	/* fixme: We should test versions here */
-	if (g_hash_table_lookup (goodfont_dict, a->name)) return;
-
-	/* Check for fontdata with afm entry */
-	afmfd = g_hash_table_lookup (font_dict, a->alias);
-	if (!afmfd) return;
-	if (!afmfd->afm_list) return;
-	afm = afmfd->afm_list->data;
-
-	/* fixme: mess with locale */
-	afmversion = (afm->version) ? atof (afm->version) : 1.0;
-
-	/* Search, whether we have same or better pfb */
-	for (l = afmfd->pfb_list; l != NULL; l = l->next) {
-		GFIFileData *fd;
-		gdouble pfbversion;
-		fd = l->data;
-		pfbversion = (fd->version) ? atof (fd->version) : 1.0;
-		/* If we have original pfb with same or higher version, we shouldn't use type1 alias */
-		if (pfbversion >= afmversion) return;
-	}
-
-	/* So there wasn't original pfb file */
-	/* Check for fontdata with pfb entry */
-	pfbfd = NULL;
-	pfb = NULL;
-	for (l = font_list; l != NULL; l = l->next) {
-		pfbfd = (GFIFontData *) l->data;
-		if (pfbfd->pfb_list) {
-			pfb = pfbfd->pfb_list->data;
-			if (!strcmp (a->alias, pfb->entry.psname)) break;
-		}
-	}
-	if (l == NULL) return;
-
-	/* fixme: mess with locale */
-	pfbversion = (pfb->version) ? atof (pfb->version) : 1.0;
-
-	for (l = pfbfd->pfb_list; l != NULL; l = l->next) {
-		GFIFileData *fd;
-		gdouble v;
-		fd = l->data;
-		v = (fd->version) ? atof (fd->version) : 1.0;
-		if (v > pfbversion) {
-			pfbversion = v;
-			pfb = fd;
-		}
-	}
-
-	/* We have to read afm to get weight and italicangle */
-
-	fi = NULL;
-	f = fopen (afm->entry.path, "r");
-	/* This shouldn't happen */
-	if (!f) return;
-	status = parseFile (f, &fi, P_G);
-	fclose (f);
-	/* This shouldn't happen! */
-	if (status != AFM_ok) {
-		if (fi) parseFileFree (fi);
-		return;
-	}
-
-	/* Now we should have everything we need */
-
-	/* Final check - test, whether given font is already registered */
-	if (gfi_font_is_registered (afmfd->name)) {
-		if (gfi_debug) fprintf (stderr, "Font %s is already registered\n", afmfd->name);
-		return;
-	}
-
-	new = g_new0 (GFFontEntry, 1);
-	new->next = NULL;
-	new->type = GF_FONT_ENTRY_TYPE1;
-	new->name = g_strdup (afmfd->name);
-	new->version = g_strdup (afm->version);
-	new->familyname = g_strdup (fi->gfi->familyName);
-	new->speciesname = gfi_get_species_name (new->name, new->familyname);
-	new->notice = (fi->gfi->notice) ? g_strdup (fi->gfi->notice) : NULL;
-	new->weight = g_strdup (fi->gfi->weight);
-	/* AFM */
-	new->files[1] = afm->entry;
-	/* PFB */
-	new->files[0] = pfb->entry;
-	/* Misc */
-	new->italicangle = fi->gfi->italicAngle;
-
-	/* Release AFM info */
-	parseFileFree (fi);
-
-	/* Register it */
-
-	if (gfi_debug) fprintf (stderr, "Registered Type1 alias: %s\n", new->name);
-
-	goodfont_list = g_slist_prepend (goodfont_list, new);
-	g_hash_table_insert (goodfont_dict, new->name, new);
-#endif
 }
 
 static void
@@ -1122,6 +1040,7 @@ gfi_build_font (GFIFontData *fd)
 		if (gfi_debug) fprintf (stderr, "Font %s is already registered\n", fd->name);
 		return;
 	}
+	if (gfi_debug) fprintf (stderr, "Building font %s\n", fd->name);
 
 	pfbdata = NULL;
 	pfbversion = -1e18;
@@ -1164,11 +1083,14 @@ gfi_build_font (GFIFontData *fd)
 	pfb = gf_pfb_open (pfbdata->entry.path);
 	if (!pfb) return;
 
+	if (gfi_debug) fprintf (stderr, "Got pfb %s\n", pfbdata->entry.path);
+	if (gfi_debug) fprintf (stderr, "Got afm %s\n", (afmdata) ? afmdata->entry.path : "NONE");
+
 	new = g_new0 (GFFontEntry, 1);
 	new->next = NULL;
 	new->type = GF_FONT_ENTRY_TYPE1;
-	new->name = g_strdup (fd->name);
-	new->version = g_strdup (pfbdata->version);
+	new->name = g_strdup (pfb->gfi.fullName);
+	new->version = g_strdup (pfb->gfi.version);
 	new->familyname = g_strdup (pfb->gfi.familyName);
 	new->speciesname = gfi_get_species_name (new->name, new->familyname);
 	new->notice = (pfb->gfi.notice) ? g_strdup (pfb->gfi.notice) : NULL;
@@ -1263,10 +1185,19 @@ static gboolean
 gfi_font_is_registered (const guchar *name)
 {
 	GFFontMap *map;
+	gint ourversion;
+
+	if (!gfi_refresh) {
+		ourversion = (gint) floor (1000.0 * atof (VERSION) + 0.5);
+	} else {
+		ourversion = G_MAXINT;
+	}
 
 	/* Step 1 - other fontmaps of same level */
 	for (map = mastermap->next; map != NULL; map = map->next) {
-		if (g_hash_table_lookup (map->fontdict, name)) return TRUE;
+		if (map->gpversion >= ourversion) {
+			if (g_hash_table_lookup (map->fontdict, name)) return TRUE;
+		}
 	}
 	/* Step 2 - next levels */
 	if (gfi_staticmap) {
@@ -1275,17 +1206,23 @@ gfi_font_is_registered (const guchar *name)
 	} else if (gfi_dynamicmap) {
 		/* Search from static maps */
 		for (map = masterdb->staticmaps; map != NULL; map = map->next) {
-			if (g_hash_table_lookup (map->fontdict, name)) return TRUE;
+			if (map->gpversion >= ourversion) {
+				if (g_hash_table_lookup (map->fontdict, name)) return TRUE;
+			}
 		}
 		return FALSE;
 	} else {
 		/* Search from dynamic maps */
 		for (map = masterdb->dynamicmaps; map != NULL; map = map->next) {
-			if (g_hash_table_lookup (map->fontdict, name)) return TRUE;
+			if (map->gpversion >= ourversion) {
+				if (g_hash_table_lookup (map->fontdict, name)) return TRUE;
+			}
 		}
 		/* Search from static maps */
 		for (map = masterdb->staticmaps; map != NULL; map = map->next) {
-			if (g_hash_table_lookup (map->fontdict, name)) return TRUE;
+			if (map->gpversion >= ourversion) {
+				if (g_hash_table_lookup (map->fontdict, name)) return TRUE;
+			}
 		}
 		return FALSE;
 	}
@@ -1301,12 +1238,16 @@ gfi_font_entry_get_by_psname (const guchar *name)
 	/* Step 0 - scan goodfont list */
 	for (l = goodfont_list; l != NULL; l = l->next) {
 		fe = (GFFontEntry *) l->data;
-		if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+		if (fe->type == GF_FONT_ENTRY_TYPE1) {
+			if (!strcmp (fe->files[0].psname, name)) return fe;
+		}
 	}
 	/* Step 1 - other fontmaps of same level */
 	for (map = mastermap->next; map != NULL; map = map->next) {
 		for (fe = map->fonts; fe != NULL; fe = fe->next) {
-			if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+			if (fe->type == GF_FONT_ENTRY_TYPE1) {
+				if (!strcmp (fe->files[0].psname, name)) return fe;
+			}
 		}
 	}
 	/* Step 2 - next levels */
@@ -1317,7 +1258,9 @@ gfi_font_entry_get_by_psname (const guchar *name)
 		/* Search from static maps */
 		for (map = masterdb->staticmaps; map != NULL; map = map->next) {
 			for (fe = map->fonts; fe != NULL; fe = fe->next) {
-				if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+				if (fe->type == GF_FONT_ENTRY_TYPE1) {
+					if (!strcmp (fe->files[0].psname, name)) return fe;
+				}
 			}
 		}
 		return FALSE;
@@ -1325,13 +1268,17 @@ gfi_font_entry_get_by_psname (const guchar *name)
 		/* Search from dynamic maps */
 		for (map = masterdb->dynamicmaps; map != NULL; map = map->next) {
 			for (fe = map->fonts; fe != NULL; fe = fe->next) {
-				if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+				if (fe->type == GF_FONT_ENTRY_TYPE1) {
+					if (!strcmp (fe->files[0].psname, name)) return fe;
+				}
 			}
 		}
 		/* Search from static maps */
 		for (map = masterdb->staticmaps; map != NULL; map = map->next) {
 			for (fe = map->fonts; fe != NULL; fe = fe->next) {
-				if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+				if (fe->type == GF_FONT_ENTRY_TYPE1) {
+					if (!strcmp (fe->files[0].psname, name)) return fe;
+				}
 			}
 		}
 	}
@@ -1375,7 +1322,7 @@ gfi_ensure_directory (const gchar *path)
 	if (!g_path_is_absolute (path)) {
 		gchar *cdir;
 		cdir = g_get_current_dir ();
-		t = g_strdup_printf ("%s/%s", cdir, path);
+		t = g_build_filename (cdir, path, NULL);
 	} else {
 		t = g_strdup (path);
 	}
@@ -1392,10 +1339,10 @@ gfi_ensure_directory (const gchar *path)
 	}
 	if (!l) return FALSE;
 	l = g_slist_reverse (l);
-	p = g_strdup ("");
+	p = g_strdup ("/");
 	while (l->next) {
 		gchar *n;
-		n = g_strdup_printf ("%s/%s", p, (gchar *) l->data);
+		n = g_build_filename (p, (gchar *) l->data, NULL);
 		l = g_slist_remove (l, l->data);
 		g_free (p);
 		p = n;
@@ -1424,6 +1371,7 @@ gfi_write_fontmap (FILE * f)
 	root = xmlNewDocNode (doc, NULL, "fontmap", NULL);
 	xmlDocSetRootElement (doc, root);
 	xmlSetProp (root, "version", "2.0");
+	xmlSetProp (root, "gpversion", VERSION);
 
 	while (goodfont_list) {
 		gfi_write_font (root, (GFFontEntry *) goodfont_list->data);
@@ -1444,7 +1392,7 @@ gfi_write_font (xmlNodePtr root, GFFontEntry *e)
 
 	if (e->type == GF_FONT_ENTRY_TYPE1) {
 		/* Set format */
-		if (e->files[0].psname && e->files[1].psname && strcmp (e->files[0].psname, e->files[1].psname)) {
+		if (e->isalias) {
 			xmlSetProp (n, "format", "type1alias");
 			xmlSetProp (n, "alias", e->files[0].psname);
 		} else {
@@ -1525,25 +1473,33 @@ gfi_get_species_name (const guchar *fullname, const guchar *familyname)
 static void
 gfi_read_aliases (const guchar *path)
 {
-	xmlDocPtr doc;
+	struct stat s;
 
-	if (gfi_debug) fprintf (stderr, "Trying alias file %s\n", path);
-
-	doc = xmlParseFile (path);
-
-	if (doc) {
-		xmlNodePtr root;
-		root = xmlDocGetRootElement (doc);
-		if (!strcmp (root->name, "fontfile")) {
-			xmlNodePtr child;
-			/* List of font entries */
-			for (child = root->xmlChildrenNode; child != NULL; child = child->next) {
-				if (!strcmp (child->name, "font")) {
-					gfi_read_alias (child);
+	if (!stat (path, &s) && S_ISREG (s.st_mode)) {
+		xmlDocPtr doc;
+		if (gfi_debug) fprintf (stderr, "Trying alias file %s ... ", path);
+		doc = xmlParseFile (path);
+		if (doc) {
+			xmlNodePtr root;
+			root = xmlDocGetRootElement (doc);
+			if (!strcmp (root->name, "fontfile")) {
+				xmlNodePtr child;
+				/* List of font entries */
+				if (gfi_debug) fprintf (stderr, "seems OK\n");
+				for (child = root->xmlChildrenNode; child != NULL; child = child->next) {
+					if (!strcmp (child->name, "font")) {
+						gfi_read_alias (child);
+					}
 				}
+			} else {
+				if (gfi_debug) fprintf (stderr, "is not font alias file\n");
 			}
+			xmlFreeDoc (doc);
+		} else {
+			if (gfi_debug) fprintf (stderr, "is not valid xml file\n");
 		}
-		xmlFreeDoc (doc);
+	} else {
+		if (gfi_debug) fprintf (stderr, "%s is not regular file\n", path);
 	}
 }
 
