@@ -29,7 +29,11 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <locale.h>
+
+#include <libart_lgpl/art_affine.h>
+
 #include "gpa-private.h"
+#include "gnome-print-private.h"
 #include "gnome-print-meta.h"
 #include "gnome-print-multipage.h"
 #include "gnome-print-master-private.h"
@@ -412,9 +416,9 @@ gnome_print_master_print (GnomePrintMaster *gpm)
 {
 	GPMPrivate *pp;
 	GnomePrintContext *ctx;
-	gint copies, nstacks, npages, nsheets;
-	gboolean collate;
-	gint s;
+	gint lpages, copies, nstacks, npages, nsheets;
+	gboolean collate, is_multipage;
+	gint stack;
 	const guchar *buf;
 	gint blen;
 	gint ret;
@@ -428,13 +432,23 @@ gnome_print_master_print (GnomePrintMaster *gpm)
 	ctx = gnome_print_context_new (gpm->config);
 	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
-	/* fixme: One DOES NOT WANT to collate multipages here */
+	/* Get number of pages in metafile */
+	lpages = gnome_print_meta_get_pages (GNOME_PRINT_META (gpm->meta));
+	if (lpages < 1) return GNOME_PRINT_OK;
+	npages = lpages;
+
+	/* Update layout data */
+	is_multipage = FALSE;
 	gpm_update_layout_data (gpm);
 	if (pp->LY_LIST) {
 		GnomePrintContext *mp;
+		/* Find out physical page count */
+		npages = (npages + pp->num_affines - 1) / pp->num_affines;
+		/* Create multipage context */
 		mp = gnome_print_multipage_new (ctx, pp->LY_LIST);
 		g_object_unref (G_OBJECT (ctx));
 		ctx = mp;
+		is_multipage = TRUE;
 	}
 
 	collate = FALSE;
@@ -450,17 +464,25 @@ gnome_print_master_print (GnomePrintMaster *gpm)
 		nsheets = copies;
 	}
 	
-	npages = gnome_print_meta_get_pages (GNOME_PRINT_META (gpm->meta));
 	buf = gnome_print_meta_get_buffer (GNOME_PRINT_META (gpm->meta));
 	blen = gnome_print_meta_get_length (GNOME_PRINT_META (gpm->meta));
 
-	for (s = 0; s < nstacks; s++) {
-		gint p;
-		for (p = 0; p < npages; p++) {
-			gint i;
-			for (i = 0; i < nsheets; i++) {
-				ret = gnome_print_meta_render_data_page (ctx, buf, blen, p, TRUE);
-				g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
+	for (stack = 0; stack < nstacks; stack++) {
+		gint page;
+		for (page = 0; page < npages; page++) {
+			gint sheet;
+			for (sheet = 0; sheet < nsheets; sheet++) {
+				gint start, i;
+				/* Render physical page */
+				start = page * pp->num_affines;
+				for (i = start; (i < (start + pp->num_affines)) && (i < lpages); i++) {
+					ret = gnome_print_meta_render_data_page (ctx, buf, blen, i, TRUE);
+					g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
+				}
+				/* Flush physical page */
+				if (is_multipage) {
+					gnome_print_multipage_finish_page (GNOME_PRINT_MULTIPAGE (ctx));
+				}
 			}
 		}
 	}
@@ -597,9 +619,9 @@ static void
 gpm_update_layout_data (GnomePrintMaster *gpm)
 {
 	GPMPrivate *pp;
-	ArtDRect area, r, PPDC;
-	gdouble xscale, yscale, scale, t;
-	gdouble PPDP2C[6], a[6], b[6];
+	ArtDRect area, r;
+	gdouble t;
+	gdouble a[6];
 	gint i;
 
 	g_return_if_fail (gpm->priv);
@@ -680,43 +702,21 @@ gpm_update_layout_data (GnomePrintMaster *gpm)
 
 	for (i = 0; i < pp->num_affines; i++) {
 		gdouble ly2p[6];
+		gdouble *ly2pa;
 		/* Calculate Layout -> Physical Page affine */
 		memcpy (ly2p, pp->affines + 6 * i, 6 * sizeof (gdouble));
 		ly2p[4] *= pp->pw;
 		ly2p[5] *= pp->ph;
 		/* PRINT_AFFINE ("Layout -> Physical:", &l2p[0]); */
 		art_affine_multiply (pp->LY_AFFINES + 6 * i, pp->LP2LY, ly2p);
-		pp->LY_LIST = g_list_prepend (pp->LY_LIST, pp->LY_AFFINES + 6 * i);
+		ly2pa = g_new (gdouble, 6);
+		art_affine_multiply (ly2pa, pp->LY_AFFINES + 6 * i, pp->PP2PA);
+		pp->LY_LIST = g_list_prepend (pp->LY_LIST, ly2pa);
 	}
 
 	pp->LY_LIST = g_list_reverse (pp->LY_LIST);
 }
 
-static gboolean
-gnome_print_config_get_transform (GnomePrintConfig *config, const guchar *key, gdouble *transform)
-{
-	guchar *v;
-	gdouble t[6];
-	gboolean ret;
-	gchar *loc;
-
-	v = gnome_print_config_get (config, key);
-	if (!v) return FALSE;
-
-	loc = setlocale (LC_NUMERIC, NULL);
-	setlocale (LC_NUMERIC, "C");
-
-	ret = gpm_parse_transform (v, t);
-	g_free (v);
-	if (ret) {
-		memcpy (transform, t, 6 * sizeof (gdouble));
-	}
-
-	setlocale (LC_NUMERIC, loc);
-
-	return ret;
-}
-
 static void
 gpm_parse_config_data (GnomePrintMaster *gpm)
 {
@@ -758,23 +758,18 @@ gpm_parse_config_data (GnomePrintMaster *gpm)
 	if (layout) {
 		gint numlp;
 		numlp = 0;
-		g_print ("Got layout\n");
 		if (gpa_node_get_int_path_value (layout, "LogicalPages", &numlp) && (numlp > 0)) {
 			GPANode *pages;
-			g_print ("Got numlp %d\n", numlp);
 			pages = gpa_node_get_path_node (layout, "Pages");
 			if (pages) {
 				GPANode *page;
 				gdouble *affines;
 				gint pagenum;
-				g_print ("Got pages\n");
 				affines = g_new (gdouble, 6 * numlp);
 				pagenum = 0;
 				for (page = gpa_node_get_child (pages, NULL); page != NULL; page = gpa_node_get_child (pages, page)) {
 					guchar *transform;
-					gboolean ret;
 					transform = gpa_node_get_value (page);
-					g_print ("Got transform value %s\n", transform);
 					gpa_node_unref (page);
 					if (!transform) break;
 					gpm_parse_transform (transform, affines + 6 * pagenum);
@@ -824,17 +819,260 @@ gpm_clear_config_data (GnomePrintMaster *gpm)
 		g_free (pp->LY_AFFINES);
 		pp->LY_AFFINES = NULL;
 	}
-	if (pp->LY_LIST) {
-		g_list_free (pp->LY_LIST);
-		pp->LY_LIST = NULL;
+	while (pp->LY_LIST) {
+		g_free (pp->LY_LIST->data);
+		pp->LY_LIST = g_list_remove (pp->LY_LIST, pp->LY_LIST->data);
 	}
 }
 
+/* All keys can be NULL */
+GnomePrintLayoutData *gnome_print_config_get_layout_data (GnomePrintConfig *config,
+							  const guchar *pagekey,
+							  const guchar *porientkey,
+							  const guchar *lorientkey,
+							  const guchar *layoutkey)
+{
+	GnomePrintLayoutData *lyd;
+	guchar key[1024];
+	const GnomePrintUnit *unit;
+	GPANode *layout;
+	gchar *loc;
+	/* Local data */
+	GnomePrintLayoutPageData *pages;
+	gdouble porient[6], lorient[6];
+	gdouble pw, ph, lyw, lyh;
+	gint num_pages;
+
+	g_return_val_if_fail (config != NULL, NULL);
+
+	if (!pagekey) pagekey = GNOME_PRINT_KEY_PAPER_SIZE;
+	if (!porientkey) porientkey = GNOME_PRINT_KEY_PAPER_ORIENTATION;
+	if (!lorientkey) lorientkey = GNOME_PRINT_KEY_PAGE_ORIENTATION;
+	if (!layoutkey) layoutkey = GNOME_PRINT_KEY_LAYOUT;
+
+
+	/* Initialize */
+	pw = 210 * 72.0 / 25.4;
+	ph = 297 * 72.0 / 25.4;
+	art_affine_identity (porient);
+	art_affine_identity (lorient);
+	lyw = 1.0;
+	lyh = 1.0;
+	num_pages = 0;
+	pages = NULL;
+
+	/* Now the fun part */
+
+	loc = setlocale (LC_NUMERIC, NULL);
+	setlocale (LC_NUMERIC, "C");
+
+	/* Physical size */
+	g_snprintf (key, 1024, "%s.Width", pagekey);
+	if (gnome_print_config_get_length (config, key, &pw, &unit)) {
+		gnome_print_convert_distance (&pw, unit, GNOME_PRINT_PS_UNIT);
+	}
+	g_snprintf (key, 1024, "%s.Height", pagekey);
+	if (gnome_print_config_get_length (config, key, &ph, &unit)) {
+		gnome_print_convert_distance (&ph, unit, GNOME_PRINT_PS_UNIT);
+	}
+	/* Physical orientation */
+	g_snprintf (key, 1024, "%s.Paper2PrinterTransform", porientkey);
+	gnome_print_config_get_transform (config, key, porient);
+	/* Logical orientation */
+	g_snprintf (key, 1024, "%s.Page2LayoutTransform", lorientkey);
+	gnome_print_config_get_transform (config, key, lorient);
+	/* Layout size */
+	g_snprintf (key, 1024, "%s.Width", layoutkey);
+	gnome_print_config_get_double (config, key, &lyw);
+	g_snprintf (key, 1024, "%s.Height", layoutkey);
+	gnome_print_config_get_double (config, key, &lyh);
+
+	/* Now come the affines */
+	layout = gpa_node_get_path_node (GNOME_PRINT_CONFIG_NODE (config), layoutkey);
+	if (layout) {
+		gint numlp;
+		numlp = 0;
+		if (gpa_node_get_int_path_value (layout, "LogicalPages", &numlp) && (numlp > 0)) {
+			GPANode *pnodes;
+			pnodes = gpa_node_get_path_node (layout, "Pages");
+			if (pnodes) {
+				GPANode *page;
+				gint pagenum;
+				pages = g_new (GnomePrintLayoutPageData, numlp);
+				pagenum = 0;
+				for (page = gpa_node_get_child (pnodes, NULL); page != NULL; page = gpa_node_get_child (pnodes, page)) {
+					guchar *transform;
+					transform = gpa_node_get_value (page);
+					gpa_node_unref (page);
+					if (!transform) break;
+					gpm_parse_transform (transform, pages[pagenum].matrix);
+					g_free (transform);
+					pagenum += 1;
+					if (pagenum >= numlp) break;
+				}
+				gpa_node_unref (pnodes);
+				if (pagenum == numlp) {
+					num_pages = numlp;
+				} else {
+					g_free (pages);
+				}
+			}
+		}
+		gpa_node_unref (layout);
+	}
+	g_return_val_if_fail (num_pages > 0, NULL);
+
+	setlocale (LC_NUMERIC, loc);
+
+	/* Success */
+
+	lyd = g_new (GnomePrintLayoutData, 1);
+	lyd->pw = pw;
+	lyd->ph = ph;
+	memcpy (lyd->porient, porient, 6 * sizeof (gdouble));
+	memcpy (lyd->lorient, lorient, 6 * sizeof (gdouble));
+	lyd->lyw = lyw;
+	lyd->lyh = lyh;
+	lyd->num_pages = num_pages;
+	lyd->pages = pages;
+
+	return lyd;
+}
+
+void
+gnome_print_layout_data_free (GnomePrintLayoutData *lyd)
+{
+	g_return_if_fail (lyd != NULL);
+
+	if (lyd->pages) g_free (lyd->pages);
+	g_free (lyd);
+}
+
+GnomePrintLayout *
+gnome_print_layout_new_from_data (const GnomePrintLayoutData *lyd)
+{
+	GnomePrintLayout *ly;
+	ArtDRect area, r;
+	gdouble t;
+	gdouble a[6];
+	gint i;
+	/* Layout data */
+	gdouble PP2PA[6], LP2LY[6];
+	gdouble PAW, PAH, LYW, LYH, LW, LH;
+
+	g_return_val_if_fail (lyd != NULL, NULL);
+	g_return_val_if_fail (lyd->num_pages > 0, NULL);
+	g_return_val_if_fail (lyd->pages != NULL, NULL);
+
+	/* Now comes the fun part */
+
+	g_return_val_if_fail ((lyd->pw > EPSILON) && (lyd->ph > EPSILON), NULL);
+
+	/* Initial setup */
+	/* Calculate PP2PA */
+	/* We allow only rectilinear setups, so we can cheat */
+	PP2PA[0] = lyd->porient[0];
+	PP2PA[1] = lyd->porient[1];
+	PP2PA[2] = lyd->porient[2];
+	PP2PA[3] = lyd->porient[3];
+	t = lyd->pw * PP2PA[0] + lyd->ph * PP2PA[2];
+	PP2PA[4] = (t < 0) ? -t : 0.0;
+	t = lyd->pw * PP2PA[1] + lyd->ph * PP2PA[3];
+	PP2PA[5] = (t < 0) ? -t : 0.0;
+	PRINT_AFFINE ("PP2PA:", &PP2PA[0]);
+
+	/* PPDP - Physical Page Dimensions in Printer */
+	/* A: PhysicalPage X PhysicalOrientation X TRANSLATE -> Physical Page in Printer */
+	area.x0 = 0.0;
+	area.y0 = 0.0;
+	area.x1 = lyd->pw;
+	area.y1 = lyd->ph;
+	art_drect_affine_transform (&r, &area, PP2PA);
+	PAW = r.x1 - r.x0;
+	PAH = r.y1 - r.y0;
+	g_return_val_if_fail ((PAW > EPSILON) || (PAH > EPSILON), NULL);
+
+	/* Now we have to find the size of layout page */
+	/* Again, knowing that layouts are rectilinear helps us */
+	art_affine_invert (a, lyd->pages[0].matrix);
+	PRINT_AFFINE ("INV LY:", &a[0]);
+	LYW = lyd->lyw * fabs (lyd->pw * a[0] + lyd->ph * a[2]);
+	LYH = lyd->lyh * fabs (lyd->pw * a[1] + lyd->ph * a[3]);
+	PRINT_2 ("LY Dimensions:", LYW, LYH);
+
+	/* Calculate LP2LY */
+	/* We allow only rectilinear setups, so we can cheat */
+	LP2LY[0] = lyd->lorient[0];
+	LP2LY[1] = lyd->lorient[1];
+	LP2LY[2] = lyd->lorient[2];
+	LP2LY[3] = lyd->lorient[3];
+	/* Delay */
+	LP2LY[4] = 0.0;
+	LP2LY[5] = 0.0;
+	/* Meanwhile find logical width and height */
+	area.x0 = 0.0;
+	area.y0 = 0.0;
+	area.x1 = LYW;
+	area.y1 = LYH;
+	art_affine_invert (a, LP2LY);
+	art_drect_affine_transform (&r, &area, a);
+	LW = r.x1 - r.x0;
+	LH = r.y1 - r.y0;
+	g_return_val_if_fail ((LW > EPSILON) && (LH > EPSILON), NULL);
+	PRINT_2 ("L Dimensions", LW, LH);
+	/* Now complete matrix calculation */
+	t = LW * LP2LY[0] + LH * LP2LY[2];
+	LP2LY[4] = (t < 0) ? -t : 0.0;
+	t = LW * LP2LY[1] + LH * LP2LY[3];
+	LP2LY[5] = (t < 0) ? -t : 0.0;
+	PRINT_AFFINE ("LP2LY:", &LP2LY[0]);
+
+	/* So we are safely here and can allocate target */
+	ly = g_new (GnomePrintLayout, 1);
+	memcpy (ly->PP2PA, PP2PA, 6 * sizeof (gdouble));
+	ly->PAW = PAW;
+	ly->PAH = PAH;
+	memcpy (ly->LP2LY, LP2LY, 6 * sizeof (gdouble));
+	ly->LYW = LYW;
+	ly->LYH = LYH;
+	ly->LW = LW;
+	ly->LH = LH;
+
+	/* Good, now generate actual layout matrixes */
+
+	ly->NLY = lyd->num_pages;
+	ly->LYP = g_new (GnomePrintLayoutPage, 6);
+
+	/* Extra fun */
+
+	for (i = 0; i < lyd->num_pages; i++) {
+		gdouble ly2p[6];
+		/* Calculate Layout -> Physical Page affine */
+		memcpy (ly2p, lyd->pages[i].matrix, 6 * sizeof (gdouble));
+		ly2p[4] *= lyd->pw;
+		ly2p[5] *= lyd->ph;
+		/* PRINT_AFFINE ("Layout -> Physical:", &l2p[0]); */
+		art_affine_multiply (ly->LYP[i].matrix, LP2LY, ly2p);
+	}
+
+	return ly;
+}
+
+void
+gnome_print_layout_free (GnomePrintLayout *layout)
+{
+	g_return_if_fail (layout != NULL);
+
+	if (layout->LYP) g_free (layout->LYP);
+	g_free (layout);
+}
+
 static gboolean
 gpm_parse_transform (guchar *str, gdouble *transform)
 {
 	gdouble t[6];
-	gchar *p, *e;
+	guchar *p;
+	gchar *e;
 	gint i;
 
 	art_affine_identity (transform);
@@ -843,7 +1081,7 @@ gpm_parse_transform (guchar *str, gdouble *transform)
 	p = strchr (str, '(');
 	if (!p) return FALSE;
 	p += 1;
-	if (!*p) return;
+	if (!*p) return FALSE;
 	for (i = 0; i < 6; i++) {
 		while (*p && isspace (*p)) p += 1;
 		if (!strncmp (p, "SQRT2", 5)) {
@@ -861,8 +1099,7 @@ gpm_parse_transform (guchar *str, gdouble *transform)
 		} else {
 			t[i] = strtod (p, &e);
 		}
-		g_print ("Got %g from %s\n", t[i], p);
-		if (e == p) return FALSE;
+		if (e == (gchar *) p) return FALSE;
 		p = e;
 	}
 
@@ -872,3 +1109,35 @@ gpm_parse_transform (guchar *str, gdouble *transform)
 	return TRUE;
 }
 
+/* fixme: Move this (and previous) to gnome-print-config.c (Lauris) */
+
+gboolean
+gnome_print_config_get_transform (GnomePrintConfig *config, const guchar *key, gdouble *transform)
+{
+	guchar *v;
+	gdouble t[6];
+	gboolean ret;
+	gchar *loc;
+
+	g_return_val_if_fail (config != NULL, FALSE);
+	g_return_val_if_fail (key != NULL, FALSE);
+	g_return_val_if_fail (*key != '\0', FALSE);
+	g_return_val_if_fail (config != NULL, FALSE);
+
+	v = gnome_print_config_get (config, key);
+	if (!v) return FALSE;
+
+	loc = setlocale (LC_NUMERIC, NULL);
+	setlocale (LC_NUMERIC, "C");
+
+	ret = gpm_parse_transform (v, t);
+	g_free (v);
+	if (ret) {
+		memcpy (transform, t, 6 * sizeof (gdouble));
+	}
+
+	setlocale (LC_NUMERIC, loc);
+
+	return ret;
+}
+
