@@ -42,15 +42,17 @@
  */
 
 static const GnomePrintUnit gp_units[] = {
+	/* Do not insert any elements before/between first 4 */
 	{0, GNOME_PRINT_UNIT_DIMENSIONLESS, 1.0, N_("Unit"), "", N_("Units"), ""},
 	{0, GNOME_PRINT_UNIT_ABSOLUTE, 1.0, N_("Point"), N_("Pt"), N_("Points"), N_("Pts")},
 	{0, GNOME_PRINT_UNIT_USERSPACE, 1.0, N_("Userspace unit"), N_("User"), N_("Userspace units"), N_("User")},
 	{0, GNOME_PRINT_UNIT_DEVICE, 1.0, N_("Pixel"), N_("Px"), N_("Pixels"), N_("Px")},
+	/* You can add new elements from this point forward */
 	{0, GNOME_PRINT_UNIT_DIMENSIONLESS, 0.01, N_("Percent"), N_("%"), N_("Percents"), N_("%")},
 	{0, GNOME_PRINT_UNIT_ABSOLUTE, (72.0 / 25.4), N_("Millimeter"), N_("mm"), N_("Millimeters"), N_("mm")},
 	{0, GNOME_PRINT_UNIT_ABSOLUTE, (72.0 / 2.54), N_("Centimeter"), N_("cm"), N_("Centimeters"), N_("cm")},
 	{0, GNOME_PRINT_UNIT_ABSOLUTE, (72.0 / 0.0254), N_("Meter"), N_("m"), N_("meters"), N_("m")},
-	{0, GNOME_PRINT_UNIT_ABSOLUTE, (72.0), N_("Inch"), N_("in"), N_("inches"), N_("in")},
+	{0, GNOME_PRINT_UNIT_ABSOLUTE, (72.0), N_("Inch"), N_("in"), N_("Inches"), N_("in")},
 };
 
 #define gp_num_units (sizeof (gp_units) / sizeof (gp_units[0]))
@@ -123,7 +125,7 @@ gnome_print_paper_get_by_name (const guchar *name)
 	if (!gp_papers) gnome_print_papers_load ();
 
 	for (l = gp_papers; l != NULL; l = l->next) {
-		if (!strcasecmp (name, ((GnomePrintPaper *) l->data)->name)) return l->data;
+		if (!g_ascii_strcasecmp (name, ((GnomePrintPaper *) l->data)->name)) return l->data;
 	}
 
 	return NULL;
@@ -246,8 +248,8 @@ gnome_print_unit_get_by_name (const guchar *name)
 	g_return_val_if_fail (name != NULL, NULL);
 
 	for (i = 0; i < gp_num_units; i++) {
-		if (!strcasecmp (name, gp_units[i].name)) return &gp_units[i];
-		if (!strcasecmp (name, gp_units[i].plural)) return &gp_units[i];
+		if (!g_ascii_strcasecmp (name, gp_units[i].name)) return &gp_units[i];
+		if (!g_ascii_strcasecmp (name, gp_units[i].plural)) return &gp_units[i];
 	}
 
 	return NULL;
@@ -261,8 +263,8 @@ gnome_print_unit_get_by_abbreviation (const guchar *abbreviation)
 	g_return_val_if_fail (abbreviation != NULL, NULL);
 
 	for (i = 0; i < gp_num_units; i++) {
-		if (!strcasecmp (abbreviation, gp_units[i].abbr)) return &gp_units[i];
-		if (!strcasecmp (abbreviation, gp_units[i].abbr_plural)) return &gp_units[i];
+		if (!g_ascii_strcasecmp (abbreviation, gp_units[i].abbr)) return &gp_units[i];
+		if (!g_ascii_strcasecmp (abbreviation, gp_units[i].abbr_plural)) return &gp_units[i];
 	}
 
 	return NULL;
@@ -304,6 +306,10 @@ gnome_print_convert_distance (gdouble *distance, const GnomePrintUnit *from, con
 	g_return_val_if_fail (from != NULL, FALSE);
 	g_return_val_if_fail (to != NULL, FALSE);
 
+	if ((from->base == GNOME_PRINT_UNIT_DIMENSIONLESS) || (to->base == GNOME_PRINT_UNIT_DIMENSIONLESS)) {
+		*distance = *distance * from->unittobase / to->unittobase;
+	}
+
 	if (from->base != to->base) return FALSE;
 
 	*distance = *distance * from->unittobase / to->unittobase;
@@ -324,6 +330,10 @@ gnome_print_convert_distance_full (gdouble *distance, const GnomePrintUnit *from
 
 	if (from->base == to->base) return gnome_print_convert_distance (distance, from, to);
 
+	if ((from->base == GNOME_PRINT_UNIT_DIMENSIONLESS) || (to->base == GNOME_PRINT_UNIT_DIMENSIONLESS)) {
+		*distance = *distance * from->unittobase / to->unittobase;
+	}
+
 	switch (from->base) {
 	case GNOME_PRINT_UNIT_ABSOLUTE:
 		absolute = *distance * from->unittobase;
@@ -349,6 +359,7 @@ gnome_print_convert_distance_full (gdouble *distance, const GnomePrintUnit *from
 	}
 
 	switch (to->base) {
+	case GNOME_PRINT_UNIT_DIMENSIONLESS:
 	case GNOME_PRINT_UNIT_ABSOLUTE:
 		*distance = absolute / to->unittobase;
 		break;
