@@ -27,6 +27,8 @@
  *
  */
 
+#include <config.h>
+
 #include <math.h>
 #include <string.h>
 #include <time.h>
@@ -54,6 +56,7 @@ struct _GPPS2Font {
 	GPPS2Font *next;
 	GnomeFontFace *face;
 	GFPSObject *pso;
+	gdouble currentsize;
 };
 
 struct _GPPS2Page {
@@ -61,7 +64,7 @@ struct _GPPS2Page {
 	gchar *name;
 	gint number;
 	gboolean shown;
-	GSList *fonts;
+	GSList *usedfonts;
 };
 
 struct _GnomePrintPS2 {
@@ -70,16 +73,16 @@ struct _GnomePrintPS2 {
 	/* List of used fonts */
 	GPPS2Font *fonts;
 	/* Active font */
-	GPPS2Font *active_font;
-	gdouble active_font_size;
+	GPPS2Font *selectedfont;
 	/* Active color */
         gdouble r, g, b;
-	gint active_color_flag;
+	gint private_color_flag;
 	/* List of pages */
 	GPPS2Page *pages;
 	/* Bookkeeping */
 	gint gsave_level;
 	gint ps_level;
+	/* Size of printed area */
 	ArtDRect bbox;
 	/* Buffer */
 	FILE *buf;
@@ -173,10 +176,9 @@ gnome_print_ps2_init (GnomePrintPS2 *ps2)
 	ps2->gsave_level = 0;
 
 	ps2->fonts = NULL;
+	ps2->selectedfont = NULL;
 
-	ps2->active_font = NULL;
-	ps2->active_font_size = 0.0;
-	ps2->active_color_flag = GP_GC_FLAG_UNSET;
+	ps2->private_color_flag = GP_GC_FLAG_UNSET;
 
 	ps2->pages = NULL;
 
@@ -207,7 +209,9 @@ gnome_print_ps2_finalize (GObject *object)
 		p = ps2->pages;
 		if (!p->shown) g_warning ("Page %d %s was not shown", p->number, p->name);
 		if (p->name) g_free (p->name);
-		while (p->fonts) p->fonts = g_slist_remove (p->fonts, p->fonts->data);
+		while (p->usedfonts) {
+			p->usedfonts = g_slist_remove (p->usedfonts, p->usedfonts->data);
+		}
 		ps2->pages = p->next;
 		g_free (p);
 	}
@@ -216,22 +220,16 @@ gnome_print_ps2_finalize (GObject *object)
 		GPPS2Font *f;
 		f = ps2->fonts;
 		if (f->face) g_object_unref (G_OBJECT (f->face));
-		if (f->pso) {
-			g_free (f->pso->buf);
-			g_free (f->pso->encodedname);
-			g_free (f->pso);
-		}
+		if (f->pso) gnome_font_face_pso_free (f->pso);
 		ps2->fonts = f->next;
 		g_free (f);
 	}
 
-	ps2->active_font = NULL;
-	ps2->active_font_size = 0.0;
-	ps2->active_color_flag = GP_GC_FLAG_UNSET;
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+#define GPPS2_BIGVAL 1e18
+
 static gint
 gnome_print_ps2_construct (GnomePrintContext *ctx)
 {
@@ -257,17 +255,28 @@ gnome_print_ps2_construct (GnomePrintContext *ctx)
 	ps2->buf = fdopen (fd, "r+");
 	ps2->bufname = tmp;
 
+	/* Set bbox to emty practical infinity */
 	ps2->bbox.x0 = 0.0;
 	ps2->bbox.y0 = 0.0;
-	ps2->bbox.x1 = 21.0 * (72.0 / 2.54);
-	ps2->bbox.y1 = 29.7 * (72.0 / 2.54);
-	/* fixme: We should not query physical, but logical page here */
+	ps2->bbox.x1 = 210 * 72.0 / 25.4;
+	ps2->bbox.y1 = 297 * 72.0 / 25.4;
 	if (gnome_print_config_get_length (ctx->config, GNOME_PRINT_KEY_PAPER_WIDTH, &ps2->bbox.x1, &unit)) {
 		gnome_print_convert_distance (&ps2->bbox.x1, unit, GNOME_PRINT_PS_UNIT);
 	}
 	if (gnome_print_config_get_length (ctx->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &ps2->bbox.y1, &unit)) {
 		gnome_print_convert_distance (&ps2->bbox.y1, unit, GNOME_PRINT_PS_UNIT);
 	}
+	if (ctx->config) {
+		gdouble pp2pa[6];
+		art_affine_identity (pp2pa);
+		if (gnome_print_config_get_transform (ctx->config, GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX, pp2pa)) {
+			art_drect_affine_transform (&ps2->bbox, &ps2->bbox, pp2pa);
+			ps2->bbox.x1 -= ps2->bbox.x0;
+			ps2->bbox.y1 -= ps2->bbox.y0;
+			ps2->bbox.x0 = 0.0;
+			ps2->bbox.y0 = 0.0;
+		}
+	}
 
 	return GNOME_PRINT_OK;
 }
@@ -292,8 +301,8 @@ gnome_print_ps2_grestore (GnomePrintContext *ctx)
 	ps2 = GNOME_PRINT_PS2 (ctx);
 
 	ps2->gsave_level -= 1;
-	ps2->active_font = NULL;
-	ps2->active_color_flag = GP_GC_FLAG_UNSET;
+	ps2->selectedfont = NULL;
+	ps2->private_color_flag = GP_GC_FLAG_UNSET;
 
 	return gp_ps2_fprintf (ps2, "Q" EOL);
 }
@@ -338,6 +347,9 @@ gnome_print_ps2_fill (GnomePrintContext *ctx, const ArtBpath *bpath, ArtWindRule
 		ret = gp_ps2_fprintf (ps2, "f*" EOL);
 	}
 
+	/* Update bbox */
+
+
 	return ret;
 }
 
@@ -458,17 +470,26 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 		ret = gp_ps2_fprintf (ps2, "%g %g m" EOL, pgl->glyphs[ps->start].x + dx, pgl->glyphs[ps->start].y + dy);
 		g_return_val_if_fail (ret >= 0, ret);
 		/* Build string */
+
+
+
 		ret = gp_ps2_fprintf (ps2, "(");
-		for (i = ps->start; i < ps->start + ps->length; i++) {
-			gint glyph;
-			glyph = pgl->glyphs[i].glyph & 0xff;
-			if (ps2->active_font->pso->encodedbytes == 1) {
-				/* 8 bit encoding */
+
+		if (ps2->selectedfont->pso->encodedbytes == 1) {
+			/* 8-bit encoding */
+			for (i = ps->start; i < ps->start + ps->length; i++) {
+				gint glyph;
+				glyph = pgl->glyphs[i].glyph & 0xff;
+				gnome_font_face_pso_mark_glyph (ps2->selectedfont->pso, glyph);
 				ret = gp_ps2_fprintf (ps2, "\\%o", glyph);
 				g_return_val_if_fail (ret >= 0, ret);
-			} else {
-				gint page;
-				/* 16 bit encoding */
+			}
+		} else {
+			/* 16-bit encoding */
+			for (i = ps->start; i < ps->start + ps->length; i++) {
+				gint glyph, page;
+				gnome_font_face_pso_mark_glyph (ps2->selectedfont->pso, pgl->glyphs[i].glyph);
+				glyph = pgl->glyphs[i].glyph & 0xff;
 				page = (pgl->glyphs[i].glyph >> 8) & 0xff;
 				ret = gp_ps2_fprintf (ps2, "\\%o\\%o", page, glyph);
 				g_return_val_if_fail (ret >= 0, ret);
@@ -495,8 +516,8 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 	if (!identity) {
 		ret = gp_ps2_fprintf (ps2, "Q" EOL);
 		g_return_val_if_fail (ret >= 0, ret);
-		ps2->active_font = NULL;
-		ps2->active_color_flag = GP_GC_FLAG_UNSET;
+		ps2->selectedfont = NULL;
+		ps2->private_color_flag = GP_GC_FLAG_UNSET;
 	}
 
 	gnome_pgl_destroy (pgl);
@@ -521,12 +542,12 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 	p->name = g_strdup (name);
 	p->number = number + 1;
 	p->shown = FALSE;
-	p->fonts = NULL;
+	p->usedfonts = NULL;
 
 	ps2->pages = p;
 
-	ps2->active_font = NULL;
-	ps2->active_color_flag = GP_GC_FLAG_UNSET;
+	ps2->selectedfont = NULL;
+	ps2->private_color_flag = GP_GC_FLAG_UNSET;
 
 	ret = gp_ps2_fprintf (ps2, "%%%%Page: %s %d" EOL, name, p->number);
 	g_return_val_if_fail (ret >= 0, ret);
@@ -564,9 +585,9 @@ gnome_print_ps2_showpage (GnomePrintContext *pc)
 
 	ps2 = GNOME_PRINT_PS2 (pc);
 
-	ps2->pages->shown = TRUE;
-	ps2->active_font = NULL;
-	ps2->active_color_flag = GP_GC_FLAG_UNSET;
+	if (ps2->pages) ps2->pages->shown = TRUE;
+	ps2->selectedfont = NULL;
+	ps2->private_color_flag = GP_GC_FLAG_UNSET;
 
 	ret = gp_ps2_fprintf (ps2, "SP" EOL);
 	g_return_val_if_fail (ret >= 0, ret);
@@ -574,10 +595,12 @@ gnome_print_ps2_showpage (GnomePrintContext *pc)
 	g_return_val_if_fail (ret >= 0, ret);
 	ret = gp_ps2_fprintf (ps2, "%%%%PageResources: procset gnome-print-procs-%s" EOL, VERSION);
 	g_return_val_if_fail (ret >= 0, ret);
-	while (ps2->pages->fonts) {
-		ret = gp_ps2_fprintf (ps2, "%%%%+ font %s" EOL, (gchar *) ps2->pages->fonts->data);
+	while (ps2->pages->usedfonts) {
+		GPPS2Font *font;
+		font = ps2->pages->usedfonts->data;
+		ret = gp_ps2_fprintf (ps2, "%%%%+ font %s" EOL, font->pso->encodedname);
 		g_return_val_if_fail (ret >= 0, ret);
-		ps2->pages->fonts = g_slist_remove (ps2->pages->fonts, ps2->pages->fonts->data);
+		ps2->pages->usedfonts = g_slist_remove (ps2->pages->usedfonts, ps2->pages->usedfonts->data);
 	}
 
 	return GNOME_PRINT_OK;
@@ -587,6 +610,7 @@ static gint
 gnome_print_ps2_close (GnomePrintContext *pc)
 {
 	GnomePrintPS2 *ps2;
+	GPPS2Font *f;
 	gchar *date;
 	gint len, ret;
 
@@ -610,25 +634,30 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	/* fixme: %%DocumentData: */
 	/* fixme: Should we use %%Extensions: and drop LanguageLevel? */
 	gnome_print_transport_printf (pc->transport, "%%%%LanguageLevel: 2" EOL);
-	gnome_print_transport_printf (pc->transport, "%%%%Pages: %d" EOL, ps2->pages->number);
+	gnome_print_transport_printf (pc->transport, "%%%%DocumentMedia: Regular %d %d 0 () ()" EOL,
+				      (gint) (ps2->bbox.x1 - ps2->bbox.x0), (gint) (ps2->bbox.y1 - ps2->bbox.y0));
+	gnome_print_transport_printf (pc->transport, "%%%%Orientation: %s" EOL,
+				      ((ps2->bbox.x1 - ps2->bbox.x0) <= (ps2->bbox.y1 - ps2->bbox.y0)) ? "Portrait" : "Landscape");
 	gnome_print_transport_printf (pc->transport, "%%%%BoundingBox: %d %d %d %d" EOL,
 				      (gint) floor (ps2->bbox.x0),
 				      (gint) floor (ps2->bbox.y0),
 				      (gint) ceil (ps2->bbox.x1),
 				      (gint) ceil (ps2->bbox.y1));
+	gnome_print_transport_printf (pc->transport, "%%%%Pages: %d" EOL, ps2->pages->number);
 	/* fixme: Orientation: */
 	gnome_print_transport_printf (pc->transport, "%%%%PageOrder: Ascend" EOL);
 	gnome_print_transport_printf (pc->transport, "%%%%Title: %s" EOL, "Document Title goes here");
 	gnome_print_transport_printf (pc->transport, "%%%%DocumentSuppliedResources: procset pnome-print-procs-%s" EOL, VERSION);
-	if (ps2->fonts) {
-		GPPS2Font *f;
-		/* %%DocumentSuppliedResources: */
-		for (f = ps2->fonts; f != NULL; f = f->next) {
-			gnome_print_transport_printf (pc->transport, "%%%%+ font %s" EOL, f->pso->encodedname);
-		}
+	/* %%DocumentSuppliedResources: */
+	for (f = ps2->fonts; f != NULL; f = f->next) {
+		gnome_print_transport_printf (pc->transport, "%%%%+ font %s" EOL, f->pso->encodedname);
 	}
 	gnome_print_transport_printf (pc->transport, "%%%%EndComments" EOL);
 	g_free (date);
+	/* Defaults */
+	gnome_print_transport_printf (pc->transport, "%%%%BeginDefaults" EOL);
+	gnome_print_transport_printf (pc->transport, "%%%%PageMedia: Regular" EOL);
+	gnome_print_transport_printf (pc->transport, "%%%%EndDefaults" EOL);
 	/* Prolog */
 	gnome_print_transport_printf (pc->transport, "%%%%BeginProlog" EOL);
 	/* Abbreviations */
@@ -650,17 +679,11 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	/* Setup */
 	gnome_print_transport_printf (pc->transport, "%%%%BeginSetup" EOL);
 	/* Download fonts */
-	while (ps2->fonts) {
-		GPPS2Font *f;
-		f = ps2->fonts;
+	for (f = ps2->fonts; f != NULL; f = f->next) {
+		gnome_font_face_pso_ensure_buffer (f->pso);
 		gnome_print_transport_printf (pc->transport, "%%%%BeginResource: font %s" EOL, f->pso->encodedname);
 		gnome_print_transport_write (pc->transport, f->pso->buf, f->pso->length);
 		gnome_print_transport_printf (pc->transport, "%%%%EndResource" EOL);
-		ps2->fonts = f->next;
-		g_free (f->pso->buf);
-		g_free (f->pso->encodedname);
-		g_free (f->pso);
-		g_free (f);
 	}
 	gnome_print_transport_printf (pc->transport, "%%%%EndSetup" EOL);
 	/* Write buffer */
@@ -747,14 +770,14 @@ gp_ps2_set_color_private (GnomePrintPS2 *ps2, gdouble r, gdouble g, gdouble b)
 
 	ctx = GNOME_PRINT_CONTEXT (ps2);
 
-	if ((ps2->active_color_flag == GP_GC_FLAG_CLEAR) && (r == ps2->r) && (g == ps2->g) && (b == ps2->b)) return 0;
+	if ((ps2->private_color_flag == GP_GC_FLAG_CLEAR) && (r == ps2->r) && (g == ps2->g) && (b == ps2->b)) return 0;
 
 	ret = gp_ps2_fprintf (ps2, "%.3g %.3g %.3g rg" EOL, r, g, b);
 
 	ps2->r = r;
 	ps2->g = g;
 	ps2->b = b;
-	ps2->active_color_flag = GP_GC_FLAG_CLEAR;
+	ps2->private_color_flag = GP_GC_FLAG_CLEAR;
 
 	return ret;
 }
@@ -762,41 +785,40 @@ gp_ps2_set_color_private (GnomePrintPS2 *ps2, gdouble r, gdouble g, gdouble b)
 static gint
 gp_ps2_set_font_private (GnomePrintPS2 *ps2, const GnomeFont *font)
 {
-	GnomePrintContext *ctx;
 	const GnomeFontFace *face;
 	GPPS2Font *f;
 	gint ret;
+	GSList *l;
 
-	ctx = GNOME_PRINT_CONTEXT (ps2);
+	if (ps2->selectedfont && (ps2->selectedfont->face == font->face) && (ps2->selectedfont->currentsize = font->size)) return 0;
 
 	face = gnome_font_get_face (font);
 
 	for (f = ps2->fonts; f != NULL; f = f->next) {
-		if (f->face == face) {
-			/* We found it */
-			if (ps2->active_font
-			    && (ps2->active_font->face == face)
-			    && (ps2->active_font_size == gnome_font_get_size (font))) {
-				return 0;
-			}
-			break;
-		}
+		if (f->face == face) break;
 	}
-	if (f == NULL) {
-		GFPSObject * pso;
-		pso = gnome_font_face_create_ps_object (face);
-		g_return_val_if_fail (pso != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	if (!f) {
+		/* No entry, so create one */
 		f = g_new (GPPS2Font, 1);
 		f->next = ps2->fonts;
 		ps2->fonts = f;
 		f->face = (GnomeFontFace *) face;
 		gnome_font_face_ref (face);
-		f->pso = pso;
+		f->pso = gnome_font_face_pso_new ((GnomeFontFace *) face, NULL);
+		g_return_val_if_fail (f->pso != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	}
+
+	for (l = ps2->pages->usedfonts; l != NULL; l = l->next) {
+		if ((GPPS2Font *) l->data == f) break;
+	}
+	if (!l) {
+		ps2->pages->usedfonts = g_slist_prepend (ps2->pages->usedfonts, f);
 	}
 
 	ret = gp_ps2_fprintf (ps2, "/%s FF %g F" EOL, f->pso->encodedname, gnome_font_get_size (font));
-	ps2->active_font = f;
-	ps2->active_font_size = gnome_font_get_size (font);
+
+	f->currentsize = font->size;
+	ps2->selectedfont = f;
 
 	return ret;
 }
