@@ -29,9 +29,9 @@ struct _GPPath {
 	guint allopen : 1;	/* All subpaths are open */
 };
 
-static gboolean sp_bpath_good (ArtBpath * bpath);
-static ArtBpath * sp_bpath_check_subpath (ArtBpath * bpath);
-static gint sp_bpath_length (ArtBpath * bpath);
+static gboolean sp_bpath_good (const ArtBpath * bpath);
+static const ArtBpath * sp_bpath_check_subpath (const ArtBpath * bpath);
+static gint sp_bpath_length (const ArtBpath * bpath);
 static gboolean sp_bpath_all_closed (const ArtBpath * bpath);
 static gboolean sp_bpath_all_open (const ArtBpath * bpath);
 
@@ -201,6 +201,8 @@ gp_path_duplicate (const GPPath * path)
 	g_return_val_if_fail (path != NULL, NULL);
 
 	new = gp_path_new_from_foreign_bpath (path->bpath);
+	g_return_val_if_fail (new != NULL, NULL);
+
 	new->x = path->x;
 	new->y = path->y;
 	new->hascpt = path->hascpt;
@@ -435,8 +437,6 @@ gp_path_close_all (const GPPath * path)
 	for (p = path->bpath; p->code != ART_END; p++) {
 		switch (p->code) {
 		case ART_MOVETO_OPEN:
-			start = p;
-			closed = FALSE;
 		case ART_MOVETO:
 			if ((!closed) && ((start->x3 != p->x3) || (start->y3 != p->y3))) {
 				d->code = ART_LINETO;
@@ -444,11 +444,12 @@ gp_path_close_all (const GPPath * path)
 				d->y3 = start->y3;
 				d++;
 			}
-			if (p->code == ART_MOVETO) closed = TRUE;
+			closed = (p->code == ART_MOVETO);
 			d->code = ART_MOVETO;
 			d->x3 = p->x3;
 			d->y3 = p->y3;
 			d++;
+			start = p;
 			break;
 		case ART_LINETO:
 		case ART_CURVETO:
@@ -702,7 +703,8 @@ gp_path_closepath (GPPath * path)
 		gp_path_lineto (path, bs->x3, bs->y3);
 	}
 
-	bs = path->bpath + path->substart;
+	bs = path->bpath + path->substart; /* NB. def_lineto can realloc bpath */
+
 	bs->code = ART_MOVETO;
 
 	path->allclosed = sp_bpath_all_closed (path->bpath);
@@ -842,9 +844,9 @@ gp_path_all_closed (const GPPath * path)
 /* Private methods */
 
 static
-gboolean sp_bpath_good (ArtBpath * bpath)
+gboolean sp_bpath_good (const ArtBpath * bpath)
 {
-	ArtBpath * bp;
+	const ArtBpath * bp;
 
 	g_return_val_if_fail (bpath != NULL, FALSE);
 
@@ -860,8 +862,8 @@ gboolean sp_bpath_good (ArtBpath * bpath)
 	return TRUE;
 }
 
-static ArtBpath *
-sp_bpath_check_subpath (ArtBpath * bpath)
+static const ArtBpath *
+sp_bpath_check_subpath (const ArtBpath * bpath)
 {
 	gint i, len;
 	gboolean closed;
@@ -900,7 +902,7 @@ sp_bpath_check_subpath (ArtBpath * bpath)
 }
 
 static gint
-sp_bpath_length (ArtBpath * bpath)
+sp_bpath_length (const ArtBpath * bpath)
 {
 	gint l;
 
