@@ -1,11 +1,24 @@
 #define __GPA_NODE_C__
 
 /*
- * GPANode
+ * This file is part of libgnomeprint 2
  *
- * Opaque handle to gnome-print configuration tree
+ * Libgnomeprint is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- * Authors:
+ * Libgnomeprint is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with the libgnomeprint; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Authors :
  *   Lauris Kaplinski <lauris@ximian.com>
  *
  * Copyright (C) 2001 Ximian, Inc.
@@ -16,6 +29,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <locale.h>
+#include "gpa-utils.h"
 #include "gpa-config.h"
 #include "gpa-node-private.h"
 
@@ -66,8 +80,8 @@ gpa_node_class_init (GPANodeClass *klass)
 					       G_SIGNAL_RUN_FIRST,
 					       G_STRUCT_OFFSET (GPANodeClass, modified),
 					       NULL, NULL,
-					       g_cclosure_marshal_VOID__VOID,
-					       G_TYPE_NONE, 0);
+					       g_cclosure_marshal_VOID__UINT,
+					       G_TYPE_NONE, 1, G_TYPE_UINT);
 
 	object_class->finalize = gpa_node_finalize;
 }
@@ -77,7 +91,6 @@ gpa_node_init (GPANode *node)
 {
 	node->parent = NULL;
 	node->next = NULL;
-	node->id = NULL;
 }
 
 static void
@@ -97,11 +110,6 @@ gpa_node_finalize (GObject *object)
 		g_object_steal_data (G_OBJECT (object), "idle_id");
 	}
 
-	if (node->id) {
-		g_free (node->id);
-		node->id = NULL;
-	}
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -116,7 +124,7 @@ gpa_node_new (GType type, const guchar *id)
 
 	node = g_object_new (type, NULL);
 
-	if (id) node->id = g_strdup (id);
+	if (id) node->qid = gpa_quark_from_string (id);
 
 	return node;
 }
@@ -126,9 +134,9 @@ gpa_node_construct (GPANode *node, const guchar *id)
 {
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
-	g_return_val_if_fail (node->id == NULL, NULL);
+	g_return_val_if_fail (!node->qid, NULL);
 
-	if (id) node->id = g_strdup (id);
+	if (id) node->qid = gpa_quark_from_string (id);
 
 	return node;
 }
@@ -187,7 +195,7 @@ gpa_node_set_value (GPANode *node, const guchar *value)
 		ret = GPA_NODE_GET_CLASS (node)->set_value (node, value);
 
 	if (ret) {
-		gpa_node_request_modified (node);
+		gpa_node_request_modified (node, GPA_NODE_MODIFIED_FLAG);
 	}
 
 	return ret;
@@ -230,15 +238,15 @@ gpa_node_lookup (GPANode *node, const guchar *path)
 /* Signal stuff */
 
 void
-gpa_node_request_modified (GPANode *node)
+gpa_node_request_modified (GPANode *node, guint flags)
 {
 	g_return_if_fail (node != NULL);
 	g_return_if_fail (GPA_IS_NODE (node));
 
-	if (!(GPA_NODE_FLAGS (node) & GPA_MODIFIED_FLAG)) {
-		GPA_NODE_SET_FLAGS (node, GPA_MODIFIED_FLAG);
+	if (!(GPA_NODE_FLAGS (node) & GPA_NODE_MODIFIED_FLAG)) {
+		GPA_NODE_SET_FLAGS (node, GPA_NODE_MODIFIED_FLAG);
 		if (node->parent) {
-			gpa_node_request_modified (node->parent);
+			gpa_node_request_modified (node->parent, flags);
 		} else {
 			guint id;
 			id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (node), "idle_id"));
@@ -251,13 +259,13 @@ gpa_node_request_modified (GPANode *node)
 }
 
 void
-gpa_node_emit_modified (GPANode *node)
+gpa_node_emit_modified (GPANode *node, guint flags)
 {
 	gpa_node_ref (node);
 
-	GPA_NODE_UNSET_FLAGS (node, GPA_MODIFIED_FLAG);
+	GPA_NODE_UNSET_FLAGS (node, GPA_NODE_MODIFIED_FLAG);
 
-	g_signal_emit (G_OBJECT (node), node_signals[MODIFIED], 0);
+	g_signal_emit (G_OBJECT (node), node_signals[MODIFIED], 0, flags);
 
 	gpa_node_unref (node);
 }
@@ -267,7 +275,7 @@ gpa_node_modified_idle_hook (GPANode *node)
 {
 	g_object_set_data (G_OBJECT (node), "idle_id", GUINT_TO_POINTER (0));
 
-	gpa_node_emit_modified (node);
+	gpa_node_emit_modified (node, 0);
 
 	return FALSE;
 }
@@ -291,18 +299,20 @@ gpa_node_unref (GPANode *node)
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 
-	g_object_ref (G_OBJECT (node));
+	g_object_unref (G_OBJECT (node));
 
 	return NULL;
 }
 
+/* fixme: remove this or make const (Lauris) */
+
 guchar *
 gpa_node_id (GPANode *node)
 {
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 
-	if (node->id) return g_strdup (node->id);
+	if (node->qid) return g_strdup (gpa_quark_to_string (node->qid));
 
 	return NULL;
 }
