@@ -1,30 +1,34 @@
 #define __GPA_VALUE_C__
 
-/*                                                            
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+/*
+ * This file is part of libgnomeprint 2
  *
- * This program is distributed in the hope that it will be useful,
+ * Libgnomeprint is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Libgnomeprint is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with the libgnomeprint; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors :
+ *   Jose M. Celorio <chema@ximian.com>
  *   Lauris Kaplinski <lauris@ximian.com>
  *
- * Copyright (C) 2001 Ximian, Inc.
+ * Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
  *
  */
 
 #include <config.h>
 #include "gpa-value.h"
-#include <libxml/xmlmemory.h>
+#include <libxml/globals.h>
 
 /* GPAValue */
 
