@@ -1,28 +1,33 @@
 #define __GPA_VENDOR_C__
 
-/*                                                            
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+/*
+ * This file is part of libgnomeprint 2
  *
- * This program is distributed in the hope that it will be useful,
+ * Libgnomeprint is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Libgnomeprint is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with the libgnomeprint; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors :
- *   Chema Celorio <chema@celorio.com>
+ *   Jose M. Celorio <chema@ximian.com>
  *   Lauris Kaplinski <lauris@ximian.com>
  *
  * Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
  *
  */
 
+#define noGPA_VENDOR_DEBUG
+
 #include <string.h>
 #include <dirent.h> /* For the DIR structure stuff */
 #include <libxml/xmlmemory.h>
@@ -46,10 +51,12 @@ static gboolean gpa_vendor_verify (GPANode *node);
 static guchar *gpa_vendor_get_value (GPANode *node);
 static GPANode *gpa_vendor_get_child (GPANode *node, GPANode *ref);
 static GPANode *gpa_vendor_lookup (GPANode *node, const guchar *path);
-static void gpa_vendor_modified (GPANode *node);
+static void gpa_vendor_modified (GPANode *node, guint flags);
 
 static GPANode *gpa_vendor_new_from_file (const gchar *filename);
 
+static GPANode *gpa_vendor_new_from_tree (xmlNodePtr tree);
+
 static GPANodeClass *parent_class = NULL;
 
 GType
@@ -119,7 +126,7 @@ gpa_vendor_verify (GPANode *node)
 
 	vendor = GPA_VENDOR (node);
 
-	if (!node->id) return FALSE;
+	if (!GPA_NODE_ID_EXISTS (node)) return FALSE;
 
 	if (!vendor->name) return FALSE;
 	if (!gpa_node_verify (vendor->name)) return FALSE;
@@ -138,7 +145,7 @@ gpa_vendor_get_value (GPANode *node)
 
 	vendor = GPA_VENDOR (node);
 
-	if (node->id) return g_strdup (node->id);
+	if (GPA_NODE_ID_EXISTS (node)) return g_strdup (GPA_NODE_ID (node));
 
 	return NULL;
 }
@@ -183,26 +190,24 @@ gpa_vendor_lookup (GPANode *node, const guchar *path)
 }
 
 static void
-gpa_vendor_modified (GPANode *node)
+gpa_vendor_modified (GPANode *node, guint flags)
 {
 	GPAVendor *vendor;
 
 	vendor = GPA_VENDOR (node);
 
 	if (vendor->name && (GPA_NODE_FLAGS (vendor->name) & GPA_MODIFIED_FLAG)) {
-		gpa_node_emit_modified (vendor->name);
+		gpa_node_emit_modified (vendor->name, 0);
 	}
 	if (vendor->url && (GPA_NODE_FLAGS (vendor->url) & GPA_MODIFIED_FLAG)) {
-		gpa_node_emit_modified (vendor->url);
+		gpa_node_emit_modified (vendor->url, 0);
 	}
 	if (vendor->models && (GPA_NODE_FLAGS (vendor->models) & GPA_MODIFIED_FLAG)) {
-		gpa_node_emit_modified (GPA_NODE (vendor->models));
+		gpa_node_emit_modified (GPA_NODE (vendor->models), 0);
 	}
 }
 
-/* Public methods */
-
-GPANode *
+static GPANode *
 gpa_vendor_new_from_tree (xmlNodePtr tree)
 {
 	GPAVendor *vendor;
@@ -271,23 +276,32 @@ static void gpa_vendor_list_load_from_dir (GPAList *vendors, const gchar *dirnam
 
 static GPAList *vendors = NULL;
 
+static void
+gpa_vendors_gone (gpointer data, GObject *gone)
+{
+#ifdef GPA_VENDOR_DEBUG
+	g_print ("GPAVendor: Vendor list %p has gone\n", gone);
+#endif
+
+	vendors = NULL;
+}
+
 GPAList *
 gpa_vendor_list_load (void)
 {
-	if (!vendors) {
-		gchar *dirname;
+	gchar *dirname;
 
-		vendors = GPA_LIST (gpa_list_new (GPA_TYPE_VENDOR, FALSE));
-		gpa_node_construct (GPA_NODE (vendors), "Vendors");
+	if (vendors) return (GPAList *) gpa_node_ref (GPA_NODE (vendors));
 
-		dirname = g_strdup_printf ("%s/%s", g_get_home_dir (), ".gnome/gnome-print-2.0/vendors");
-		gpa_vendor_list_load_from_dir (vendors, dirname);
-		g_free (dirname);
-		/* fixme: */
-		gpa_vendor_list_load_from_dir (vendors, DATADIR "/gnome-print-2.0/vendors");
-	}
+	vendors = GPA_LIST (gpa_list_new (GPA_TYPE_VENDOR, FALSE));
+	gpa_node_construct (GPA_NODE (vendors), "Vendors");
+	g_object_weak_ref (G_OBJECT (vendors), gpa_vendors_gone, &vendors);
 
-	gpa_node_ref (GPA_NODE (vendors));
+	dirname = g_strdup_printf ("%s/%s", g_get_home_dir (), ".gnome/gnome-print-2.0/vendors");
+	gpa_vendor_list_load_from_dir (vendors, dirname);
+	g_free (dirname);
+	/* fixme: */
+	gpa_vendor_list_load_from_dir (vendors, DATADIR "/gnome-print-2.0/vendors");
 
 	return vendors;
 }
@@ -330,186 +344,29 @@ gpa_vendor_list_load_from_dir (GPAList *vendors, const gchar *dirname)
 	}
 }
 
+/* Public methods */
+
 GPANode *
 gpa_vendor_get_by_id (const guchar *id)
 {
+	GPAList *vendors;
 	GPANode *child;
 
 	g_return_val_if_fail (id != NULL, NULL);
 	g_return_val_if_fail (*id != '\0', NULL);
 
-	if (!vendors) {
-		GPAList *list;
-		list = gpa_vendor_list_load ();
-		gpa_node_unref (GPA_NODE (list));
-	}
+	vendors = gpa_vendor_list_load ();
 
 	for (child = vendors->children; child != NULL; child = child->next) {
 		g_assert (GPA_IS_VENDOR (child));
-		if (!strcmp (id, child->id)) {
-			gpa_node_ref (child);
-			return child;
-		}
+		if (GPA_NODE_ID_COMPARE (child, id)) break;
 	}
 
-	return NULL;
+	if (child) gpa_node_ref (child);
+
+	gpa_node_unref (gpa_node_cache (GPA_NODE (vendors)));
+
+	return child;
 }
 
-#if 0
-static GPAList *
-gpa_vendor_list_new_from_file (const gchar *filename)
-{
-	GPAList *vendors;
-	xmlDocPtr doc;
-	xmlNsPtr  name_space;
-	XmlParseContext *context;
-
-	g_return_val_if_fail (filename != NULL, NULL);
-
-	/* Laad the xml file, and check for success*/
-	doc = xmlParseFile (filename);
-	if (doc == NULL) {
-		gpa_error ("File not found [%s] (gpa-vendors)", filename);
-		return NULL;
-	}
-	if (doc->root == NULL) {
-		gpa_error ("Invalid xml File, tree empty [%s]", filename);
-		xmlFreeDoc (doc);
-		return NULL;
-	}
-
-	/* Is the xml file the correct type ? */
-	name_space = xmlSearchNsByHref (doc, doc->root, GPA_TAG_INDEX_NAME_SPACE);
-	if (name_space == NULL) {
-		gpa_error ("This is not a Printer Index file, did not contained a name_space [%s]", filename);
-
-		xmlFreeDoc (doc);
-		return NULL;
-	}
-	/* Is this a printer index file ? */
-	if ((doc->root->name == NULL) || (strcmp (doc->root->name, GPA_TAG_INDEX)!=0)) {
-		gpa_error ("This is not a PrinterIndex file, bad root name,\n"
-			   "Expected \"%s\", root name \"%s\"[%s]",
-			   GPA_TAG_INDEX,
-			   doc->root->name,
-			   filename);
-		xmlFreeDoc (doc);
-		return NULL;
-	}
-	
-	context = gpa_xml_parse_context_new (doc, name_space);
-
-	vendors = gpa_vendor_list_load_from_tree (context, doc->root);
-
-	gpa_xml_parse_context_destroy (context);
-
-	xmlFreeDoc (doc);
-
-	return vendors;
-}
-
-static GPAList *
-gpa_vendor_list_load_from_tree (XmlParseContext *context, xmlNodePtr tree)
-{
-	GPAList *vendors;
-	GPAVendor *vendor;
-	xmlNodePtr child;
-	xmlNodePtr node;
-	
-	if (strcmp (tree->name, GPA_TAG_INDEX)) {
-		gpa_error ("Invalid type in load_from_node."
-				  "Expected " GPA_TAG_INDEX );
-		return NULL;
-	}
-
-	child = gpa_xml_search_child (tree, GPA_TAG_VENDORS);
-	if (child == NULL) {
-		gpa_error ("The file did not contained any Vendors");
-		return NULL;
-	}
-
-	vendors = (GPAList *) gpa_list_new ("vendors", GPA_TYPE_VENDOR);
-	g_return_val_if_fail (vendors != NULL, NULL);
-
-	node = child->childs;
-	while (node != NULL) {
-		vendor = gpa_vendor_new_from_tree (context, node);
-		if (vendor) gpa_node_add_child (GPA_NODE (vendors), GPA_NODE (vendor), NULL);
-		node = node->next;
-	}
-
-	return vendors;
-}	
-#endif
-
-#if 0
-GPAVendor *
-gpa_vendor_list_get_vendor_by_id (GPAList *vendors, const gchar *id)
-{
-	GPAItem *item;
-
-	g_return_val_if_fail (vendors != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_LIST (vendors), NULL);
-	g_return_val_if_fail (id != NULL, NULL);
-
-	for (item = vendors->items; item != NULL; item = item->next) {
-		GPAVendor *vendor;
-		g_return_val_if_fail (GPA_IS_VENDOR (item), NULL);
-		vendor = GPA_VENDOR (item);
-		if (!strcmp (GPA_VENDOR_ID (vendor), id)) {
-			gpa_node_ref (GPA_NODE (vendor));
-			return vendor;
-		}
-	}
-
-	return NULL;
-}
-
-/*
- * Original stuff
- */
-
-GPAVendor *
-gpa_vendor_get_by_name (const gchar *name)
-{
-	GPAItem *item;
-
-	g_return_val_if_fail (name != NULL, NULL);
-
-	for (item = GPA->vendors->items; item != NULL; item = item->next) {
-		GPAVendor *vendor;
-		g_assert (GPA_IS_VENDOR (item));
-		vendor = GPA_VENDOR (item);
-		if (!strcmp (vendor->name, name)) {
-			gpa_node_ref (GPA_NODE (vendor));
-			return vendor;
-		}
-	}
-
-	return NULL;
-}
-		
-GPAModel *
-gpa_vendor_get_model_by_id (GPAVendor *vendor, const gchar *id)
-{
-	GPAItem *item;
-
-	g_return_val_if_fail (vendor != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_VENDOR (vendor), NULL);
-	g_return_val_if_fail (id != NULL, NULL);
-	g_return_val_if_fail (*id != '\0', NULL);
-
-	for (item = vendor->models->items; item != NULL; item = item->next) {
-		GPAModel *model;
-		g_assert (GPA_IS_MODEL (item));
-		model = GPA_MODEL (item);
-		if (!strcmp (GPA_ITEM (model)->id, id)) {
-			gpa_node_ref (GPA_NODE (model));
-			return model;
-		}
-	}
-
-	return NULL;
-}
-#endif
 
